@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# $Id: StatisticsHandler.py 9703 2016-08-19 10:37:44Z Judy $
+# $Id: StatisticsHandler.py 10079 2017-07-03 13:52:00Z David $
 #
 # Copyright (c) 2012 Nuwa Information Co., Ltd, All Rights Reserved.
 #
@@ -11,9 +11,9 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 #
-# $Author: Judy $
-# $Date: 2016-08-19 18:37:44 +0800 (五, 19  8 2016) $
-# $Revision: 9703 $
+# $Author: David $
+# $Date: 2017-07-03 21:52:00 +0800 (Mon, 03 Jul 2017) $
+# $Revision: 10079 $
 
 from datetime import timedelta, datetime
 
@@ -25,7 +25,6 @@ from Iuppiter.Encoding import _unicode
 
 from Theophrastus.models import Newsletter
 from Theophrastus.models import ContactMailingStatus
-from collections import OrderedDict
 
 _STATUS = [
     'deliverie', 'error', 'invaild', 'open', 'openOnSite', 'click',
@@ -36,36 +35,49 @@ STATUS_DICT = dict(ContactMailingStatus.STATUS_CHOICES)
 STATUS_NAMES = STATUS_DICT.values()
 STATUS_CODE = STATUS_DICT.keys()
 
-def getDateRange(startDate, endDate):
+def getDateRange(rangeName='week', defaultStart=None):
     """
     Get range of dates.
+
+    week: from last week to today.
+    month: from last months to today.
+    3month: from last three months to today.
+
+    @param rangeName The range of name.
     """
-    if isinstance(startDate, str):
-        if startDate != "NaN":
-            startDate = datetime.strptime(startDate, '%Y/%m/%d')
-    if isinstance(endDate, str):
-        if endDate != "NaN":
-            endDate = datetime.strptime(endDate, '%Y/%m/%d')
-    
-    deltaDays = endDate - startDate
-    daysAmount = deltaDays.days
-    
-    def _createDate(daysAmount):
-        days = [startDate.strftime('%Y-%m-%d')]
-        for i in range(1, daysAmount):
-            d = startDate + timedelta(days=i)
+    if not rangeName.strip():
+        rangeName = 'week'
+
+    endDate = datetime.now()
+
+    rangeMapping = {
+        'week': 7,
+        'month': int(1 * (365 / 12)),
+        '3month': int(3 * (365 / 12)),
+    }
+
+    if rangeName in rangeMapping:
+        deltaNumber = rangeMapping[rangeName]
+        days = [endDate.strftime('%Y-%m-%d')]
+        for i in range(1, deltaNumber):
+            d = endDate - timedelta(days=i)
+            if defaultStart and defaultStart > d:
+                break
+
             d = d.strftime('%Y-%m-%d')
             days.append(d)
 
+        days.reverse()
+
         class DateRange(object):
             def __init__(self, days):
                 self._days = days
                 self.startDate = '%s 00:00:00' % days[0]
                 self.endDate = '%s 23:59:59' % days[-1]
-                
-        return DateRange(days)
 
-    return _createDate(daysAmount)
+        return DateRange(days)
+    else:
+        return None
 
 #def countStatus(queryset):
 #    """
@@ -255,9 +267,6 @@ def getEmptyDataStructure():
         'reject': 0,
     }
 
-def getEmptyDataList():
-    return []
-
 def countStatus(queryset):
     """
     Calculate the status number of occurrences.
@@ -287,10 +296,6 @@ def countStatus(queryset):
     clickLogs = set()
     unsubscriptionNumber = set()
 
-    openId = []
-    openOnSiteId = []
-    clickId = []
-
     bouncedLogs = set()
     complainLogs = set()
     rejectedLogs = set()
@@ -305,106 +310,55 @@ def countStatus(queryset):
         elif status == ContactMailingStatus.INVALID:
             invaildLogs.add(id)
         elif status == ContactMailingStatus.UNSUBSCRIPTION:
-            unsubscriptionNumber.add(id)        
-        elif status == ContactMailingStatus.OPENED:
-            openLogs.add(id)
-            openId.append(id)     
-        elif status == ContactMailingStatus.OPENED_ON_SITE:
-            openOnSiteLogs.add(id)
-            openOnSiteId.append(id)
-            if not id in openLogs:
-                openLogs.add(id)
-                openId.append(id)              
-        elif status == ContactMailingStatus.LINK_OPENED:
-            clickLogs.add(id)
-            clickId.append(id)
-            if not id in openLogs:
-                openLogs.add(id)
-                openId.append(id)  
+            unsubscriptionNumber.add(id)
         ##
-        elif status == ContactMailingStatus.BOUNCE:
+        elif status == ContactMailingStatus.BOUNCED:
             bouncedLogs.add(id)
-        elif status == ContactMailingStatus.COMPLAIN:
+        elif status == ContactMailingStatus.COMPLAINT:
             complainLogs.add(id)
-        elif status == ContactMailingStatus.REJECT:
+        elif status == ContactMailingStatus.REJECTED:
             rejectedLogs.add(id)
     
     # formula reference: emencia.django.utils.statistics.py
     # openings
-#     openings = queryset.filter(Q(status=ContactMailingStatus.OPENED) | 
-#                                Q(status=ContactMailingStatus.OPENED_ON_SITE))
-#     openingsByLinks = len(
-#         set(queryset.filter(status=ContactMailingStatus.LINK_OPENED).exclude(
-#         contact__in=openings.values_list('contact', flat=True)
-#     ).values_list('contact', flat=True)))
-#     uniqueOpenings = len(set(openings.values_list('contact', flat=True))) + \
-#                      openingsByLinks
-#                           
-#     # on site openings
-#     onSiteOpenings = queryset.filter(
-#        status=ContactMailingStatus.OPENED_ON_SITE)
-#     uniqueOnSiteOpenings = len(set(onSiteOpenings.values_list(
-#                               'contact', flat=True)))
-#             
-#     # clicked
-#     clickedLinks = queryset.filter(status=ContactMailingStatus.LINK_OPENED)
-#     uniqueClicked = len(set(clickedLinks.values_list(
-#                               'contact', flat=True)))
-    return [{
+    openings = queryset.filter(Q(status=ContactMailingStatus.OPENED) | 
+                               Q(status=ContactMailingStatus.OPENED_ON_SITE))
+    openingsByLinks = len(
+        set(queryset.filter(status=ContactMailingStatus.LINK_OPENED).exclude(
+        contact__in=openings.values_list('contact', flat=True)
+    ).values_list('contact', flat=True)))
+    uniqueOpenings = len(set(openings.values_list('contact', flat=True))) + \
+                     openingsByLinks
+                          
+    # on site openings
+    onSiteOpenings = queryset.filter(
+       status=ContactMailingStatus.OPENED_ON_SITE)
+    uniqueOnSiteOpenings = len(set(onSiteOpenings.values_list(
+                              'contact', flat=True)))
+            
+    # clicked
+    clickedLinks = queryset.filter(status=ContactMailingStatus.LINK_OPENED)
+    uniqueClicked = len(set(clickedLinks.values_list(
+                              'contact', flat=True)))
+
+    return {
         'sentTest': len(sentTestNumber),
         'deliverie': len(sentNumber),
         'error': len(errorLogs),
         'invaild': len(invaildLogs),
-        'open': len(openLogs),
-        'openOnSite': len(openOnSiteLogs),
-        'click': len(clickLogs),
+        'open': uniqueOpenings,
+        'openOnSite': uniqueOnSiteOpenings,
+        'click': uniqueClicked,
         'unsubscribe': len(unsubscriptionNumber),
         'bounce': len(bouncedLogs),
         'complain': len(complainLogs),
-        'reject': len(rejectedLogs),        
-        
-        },
-        {
-         'openId': openId,
-         'openOnSiteId': openOnSiteId,
-         'clickId': clickId
-        }            
-    ]
+        'reject': len(rejectedLogs),
+    }
 
 from django.db.models import Q
 
 from emencia.django.newsletter.models import ContactMailingStatus as Status
 
-def getStartDateEndDate():
-    startDate = ContactMailingStatus.objects.order_by('creation_date')[0
-                ].creation_date.strftime('%Y-%m-%d')
-    endDate = ContactMailingStatus.objects.order_by('-creation_date')[0
-               ].creation_date.strftime('%Y-%m-%d')
-    return startDate, endDate
-
-def getAllDataDate():    
-    firstDate = ContactMailingStatus.objects.order_by('creation_date')[0
-                ].creation_date
-    lastDate = ContactMailingStatus.objects.order_by('-creation_date')[0
-               ].creation_date
-    
-    deltaDays= lastDate - firstDate
-    daysAmount = deltaDays.days       
-    
-    days = [firstDate.strftime('%Y-%m-%d')]
-    for i in range(1, daysAmount):
-        d = firstDate + timedelta(days=i)
-        d = d.strftime('%Y-%m-%d')
-        days.append(d)
-
-    class DateRange(object):
-        def __init__(self, days):
-            self._days = days
-            self.startDate = '%s 00:00:00' % days[0]
-            self.endDate = '%s 23:59:59' % days[-1]
-            
-    return DateRange(days)
-
 def getStatisticsTotalData(reqestGetData, subscriberCount, testSubscriberCount, 
                            conditions):
     """
@@ -414,20 +368,15 @@ def getStatisticsTotalData(reqestGetData, subscriberCount, testSubscriberCount,
     @param slug Newsletter slug.
     @param request Request instance.
     """
-    
+    period = reqestGetData.get('period', 'week')
     dataType = reqestGetData.get('dataType', 'number')
-    mode = reqestGetData.get('mode', 'accumulate')
-    allDataDates = getAllDataDate()
-    
-#     period = reqestGetData.get('period', 'week')
-# 
-#     _firstCreationDate = ContactMailingStatus.objects.filter(**conditions)
-#     _s = {"d": "strftime('%%Y-%%m-%%d', creation_date)"}
-#     _firstCreationDate = _firstCreationDate.extra(_s)
-#     _firstCreationDate = _firstCreationDate.values('d').distinct().order_by('d')
-#     _t = datetime.strptime(_firstCreationDate[0]['d'], '%Y-%m-%d')
-#     dates = getDateRange(period, defaultStart=_t)
-    
+
+    _firstCreationDate = ContactMailingStatus.objects.filter(**conditions)
+    _s = {"d": "strftime('%%Y-%%m-%%d', creation_date)"}
+    _firstCreationDate = _firstCreationDate.extra(_s)
+    _firstCreationDate = _firstCreationDate.values('d').distinct().order_by('d')
+    _t = datetime.strptime(_firstCreationDate[0]['d'], '%Y-%m-%d')
+    dates = getDateRange(period, defaultStart=_t)
 
     def getContactStatus(conditions):
         contactStatus = ContactMailingStatus.objects.filter(**conditions)
@@ -442,16 +391,9 @@ def getStatisticsTotalData(reqestGetData, subscriberCount, testSubscriberCount,
         return contactStatus
 
     def sortByDatetime(contactStatus):
-        completed = OrderedDict()
-        
-        dates = []
-        for c in contactStatus:
-            date = c['d']            
-            if not date in dates:
-                dates.append(date)
-        dates = sorted(dates)
-        
-        for date in dates:
+        completed = {}
+        for r in contactStatus:
+            date = r['d']
             year = date.split('-')[0]
             month = date.split('-')[1]
             day = date.split('-')[2]
@@ -461,10 +403,8 @@ def getStatisticsTotalData(reqestGetData, subscriberCount, testSubscriberCount,
             completed[date] = queryset
         return completed
 
-    # -- get all date content -------------------------------------------------
     conditions.update(
-        {'creation_date__range': [allDataDates.startDate, 
-                                  allDataDates.endDate]})
+        {'creation_date__range': [dates.startDate, dates.endDate]})
     
     if 'newsletter__id__in' in conditions:
         newsletterIds = conditions.pop('newsletter__id__in')
@@ -473,7 +413,7 @@ def getStatisticsTotalData(reqestGetData, subscriberCount, testSubscriberCount,
         for _id in newsletterIds:
             conditions.update({'newsletter__id': _id})
             _cs = getContactStatus(conditions)
-            _statusCount = countStatus(_cs)[0]
+            _statusCount = countStatus(_cs)
 
         # Merge.
         totalData = []
@@ -491,62 +431,20 @@ def getStatisticsTotalData(reqestGetData, subscriberCount, testSubscriberCount,
         contactStatus = getContactStatus(conditions)
         completed = sortByDatetime(contactStatus)
         totalData = []
-        openIds = []
-        openOnSiteIds = []
-        linkIds = []
         for c in completed:
-            _d = countStatus(completed[c])           
-            open = _d[1]['openId']
-            openOnSite = _d[1]['openOnSiteId']
-            link = _d[1]['clickId']
-            
-            openCount = 0
-            openOnSiteCount = 0
-            linkCount = 0
-            
-            for id in open:
-                if not id in openIds:
-                    if not id in openOnSiteIds:
-                        if not id in linkIds:
-                            openIds.append(id)
-                            openCount += 1
-                            
-            for id in openOnSite:
-                if not id in openOnSiteIds:
-                    openOnSiteIds.append(id)
-                    openOnSiteCount += 1
-                    
-            for id in link:
-                if not id in linkIds:
-                    linkIds.append(id)
-                    linkCount += 1
-                    
-            data = _d[0]
-            data['open'] = openCount
-            data['openOnSite'] = openOnSiteCount
-            data['click'] = linkCount
+            data = countStatus(completed[c])
             totalData.append([c, data])
 
-    allDates = sorted(dict(totalData).keys())
-    for day in allDataDates._days:        
+    allDates = dict(totalData).keys()
+    for day in dates._days:
         if day not in allDates:
-            OrderedDict(totalData.append([day, getEmptyDataStructure()]))  
+            totalData.append([day, getEmptyDataStructure()])
 
-    allContent = {}
+    content = {}
     if dataType == 'number':
-        #allContent[day] = [(status, value), ...]
-        if mode == 'single':
-            for (day, data) in totalData:
-                allContent[day] = [(i, data[i]) for i in data]
-        else:
-            accumulativeData = getEmptyDataStructure()
-            for (day, data) in totalData:
-                result = []
-                for status in data:
-                    accumulativeData[status] += data[status]
-                    result.append((status, accumulativeData[status]))
-                allContent[day] = result
-        
+        #content[day] = [(status, value), ...]
+        for (day, data) in totalData:
+            content[day] = [(i, data[i]) for i in data]
     else:
         def _do(info, subscriberCount, testSubscriberCount):
             rates = []
@@ -566,83 +464,25 @@ def getStatisticsTotalData(reqestGetData, subscriberCount, testSubscriberCount,
                     value = percentage(info[i], totalValue, showPercent=False)
                 
                 rates.append((i, value))
+#                 if i == 'unsubscribe':
+#                     totalValue = subscriberCount
+#                 else:
+#                     totalValue = info['deliverie']
+#                 value = percentage(info[i], totalValue, showPercent=False)
+#                 rates.append((i, value)) # (status, value)
             return rates
-        
-        if mode == 'single':
-            for (day, data) in totalData:
-                result = _do(data, subscriberCount, testSubscriberCount)
-                allContent[day] = result # {'day': (status, value)}
-        else:
-            accumulativeData = getEmptyDataStructure()
 
-            for (day, data) in totalData:
-                result = _do(data, subscriberCount, testSubscriberCount)
-                accResult = []
-                for r in result:
-                    status = r[0]
-                    value = r[1]
-                    accumulativeData[status] += value
-                    _result = (status, accumulativeData[status])
-                    accResult.append(_result)
-                allContent[day] = accResult
-    # -- get all date content end ----------------------------------------------
-    
-    # -- get range date content start ------------------------------------------
-    deltaDays = None
-    startDate = reqestGetData.get('startDate', None)    
-    startDate = startDate.encode('ascii','ignore')
-    
-    endDate = reqestGetData.get('endDate', None)
-    endDate = endDate.encode('ascii','ignore')
-    
-    period = reqestGetData.get('period', 'week')    
-    if period == 'week':
-        deltaDays = 7
-        endDate = datetime.now()        
-        startDate = endDate - timedelta(days=deltaDays)
-    elif period == 'month':
-        deltaDays = int(1 * (365 / 12))
-        endDate = datetime.now()
-        startDate = endDate - timedelta(days=deltaDays)
-    elif period == '3month':
-        deltaDays = int(3 * (365 / 12))
-        endDate = datetime.now()
-        startDate = endDate - timedelta(days=deltaDays)
-    dateRange = getDateRange(startDate, endDate)
-    
-    content = {}
-    _days = dateRange._days
-    lastDate = allDates[-1]
-    lastDateObj = datetime.strptime(allDates[-1], '%Y-%m-%d')
-    for _d in _days:
-        if _d in allContent.keys():
-            content[_d] = allContent[_d]
-        else:
-            _emptyData = getEmptyDataStructure()            
-            if mode == 'single':    
-                if dataType == 'number':
-                    content[_d] = [(i, _emptyData[i]) for i in _emptyData]
-                else:
-                    content[_d] = [(i, 0.0) for i in _emptyData]
-            else:
-                _dDateObj = datetime.strptime(_d, '%Y-%m-%d')
-                if _dDateObj > lastDateObj:
-                    content[_d] = allContent[lastDate]
-                else:
-                    if dataType == 'number':
-                        content[_d] = [(i, _emptyData[i]) for i in _emptyData]
-                    else:
-                        content[_d] = [(i, 0.0) for i in _emptyData]              
-   # -- get range date content end --------------------------------------------
-    
-    
+        for (day, data) in totalData:
+            result = _do(data, subscriberCount, testSubscriberCount)
+            content[day] = result # {'day': (status, value)}
+
     context = {}
     for day in content:
         for (statusName, value) in content[day]:
             dData = context.get(statusName, [])
             dData.append((day, value))
             context[statusName] = dData
-                
+
     # Find the max value of Y-axis.
     tmp = []
     for k in context:
@@ -657,8 +497,8 @@ def getStatisticsTotalData(reqestGetData, subscriberCount, testSubscriberCount,
             'email': context,
         },
         'rateType': dataType,
-        'dateRange': dateRange._days,
-        'xaxis': {'minDate': dateRange._days[0], 'maxDate': dateRange._days[-1]},
+        'dateRange': dates._days,
+        'xaxis': {'minDate': dates._days[0], 'maxDate': dates._days[-1]},
         'yaxis': {'max': yaxisMaxValue},
         'status': _STATUS,
         'settings': {
@@ -696,7 +536,7 @@ def getStatisticsTotalRateData(reqestGetData, subscriberCount,
             conditions.update({'newsletter__id': _id})
             _cs = getContactStatus(conditions)
             
-            _data = countStatus(_cs)[0]
+            _data = countStatus(_cs)
             for _d in _data:
                 v = _data[_d]
                 if _d not in data:
@@ -704,7 +544,7 @@ def getStatisticsTotalRateData(reqestGetData, subscriberCount,
                 data[_d] += v
     else:
         contactStatus = getContactStatus(conditions)
-        data = countStatus(contactStatus)[0]
+        data = countStatus(contactStatus)
 
     rates = []
     
